/*
 * palmjup.c
 * Main routines for Palm jupiter program.
 *
 * Copyright 2002 by Akkana Peck.
 * You are free to use or modify this code under the Gnu Public License.
 */

#include "shallow.h"
#include "palmgrfx.h"

#include "juprsc.h"

#define XWinSize 160
#define YWinSize 160
#define YWinFake 120

double WIDESCALE = .03;
double NARROWSCALE = .3;

double Scale = .03;

static int flipNS=0, flipEW=0;

extern void DrawJupiter(int Xmax, int Ymax, double Scale,
                        int flipNS, int flipEW, int color,
                        RGBColorType* fgC, RGBColorType* bgC,
                        RGBColorType* spotC, RGBColorType* bandC,
                        RGBColorType* diskC);

void RedrawScreen()
{
    ClearArea(0, 0, XWinSize, YWinFake);
    ChangeColor(&black);
    DateLabel(5, YWinSize-15, julianDate);
    DrawJupiter(XWinSize, YWinFake, Scale, flipNS, flipEW, 0,
                &black, &white, 0, 0, 0);
}

void FatalError(char* s)
{
    FrmCustomAlert(fatalAlert, s, 0, 0);
}

Boolean HandleCtlSelectEvent(EventPtr event)
{
    Boolean handled = true;
    unsigned long seconds;
    DateTimeType datetime;
    Word controlID = event->data.ctlSelect.controlID;

    switch(controlID)
    {
      case yearUp:
          seconds = JulianToPalmTime(julianDate);
          TimSecondsToDateTime(seconds, &datetime);
          ++datetime.year;
          seconds = TimDateTimeToSeconds(&datetime);
          julianDate = PalmTimeToJulian(seconds);
          break;
      case yearDown:
          seconds = JulianToPalmTime(julianDate);
          TimSecondsToDateTime(seconds, &datetime);
          --datetime.year;
          seconds = TimDateTimeToSeconds(&datetime);
          julianDate = PalmTimeToJulian(seconds);
          break;
      case dayUp:
          julianDate += 1;
          break;
      case dayDown:
          julianDate -= 1;
          break;
      case monUp:
          seconds = JulianToPalmTime(julianDate);
          TimSecondsToDateTime(seconds, &datetime);
          ++datetime.month;
          seconds = TimDateTimeToSeconds(&datetime);
          julianDate = PalmTimeToJulian(seconds);
          break;
      case monDown:
          seconds = JulianToPalmTime(julianDate);
          TimSecondsToDateTime(seconds, &datetime);
          --datetime.month;
          seconds = TimDateTimeToSeconds(&datetime);
          julianDate = PalmTimeToJulian(seconds);
          break;
      case minUp:
          julianDate += 1./24./6.;
          break;
      case minDown:
          julianDate -= 1./24./6.;
          break;
      case hourUp:
          julianDate += 1./24.;
          break;
      case hourDown:
          julianDate -= 1./24.;
          break;
      case tzUp:
          TimeZone += 1;
          TZadjustment += 1./24.;
          //julianDate += 1./24.;
          break;
      case tzDown:
          TimeZone -= 1;
          TZadjustment -= 1./24.;
          //julianDate -= 1./24.;
          break;
      case zoomBtn:
          if (Scale == WIDESCALE) {
              /*const ControlPtr btn;*/
              CtlSetLabel(event->data.ctlSelect.pControl, "z--m");
              Scale = NARROWSCALE;
          }
          else {
              CtlSetLabel(event->data.ctlSelect.pControl, "zOOm");
              Scale = WIDESCALE;
          }
          handled = 1;
          break;
      default:
          handled = false;
          break;
    }
    if (handled)
        RedrawScreen();
    return handled;
}

Boolean HandleMenuEvent(Word itemID)
{
    Boolean handled = false;
    switch (itemID)
    {
      case menuHelpAbout:
          FrmAlert(aboutAlert);
          handled = true;
          break;
      case menuHelpHelp:
          FrmAlert(helpAlert);
          handled = true;
          break;
      case menuViewFlipNS:
          flipNS = !flipNS;
          RedrawScreen();
          break;
      case menuViewFlipEW:
          flipEW = !flipEW;
          RedrawScreen();
          break;
      default:
    }
    return handled;
}

Boolean HandleEvent(EventPtr event)
{
    FormType *pfrm;
    Boolean handled = false;
    static int penX = 0;
    int penY, dX, Yc;
    static double timeOffset = .5/24.;

#ifdef __GNUC__
    CALLBACK_PROLOGUE
#endif

    switch (event->eType)
    {
      case menuEvent:
          penX = -1;
          handled = HandleMenuEvent(event->data.menu.itemID);
          break;

      case ctlSelectEvent:
          penX = -1;
          handled = HandleCtlSelectEvent(event);
          break;

      case penDownEvent:
          if (event->screenY > YWinFake)
              break;
          penX = event->screenX;
          handled = true;
          break;

      case penUpEvent:
          penY = event->screenY;
          if (penX < 0 || penY > YWinFake)   // at start, or after UI event
          {
              penX = 0;
              //printf("penX was negative, ignoring\n");
              break;
          }
          if (penX == 0)
          {
              RedrawScreen();
              handled = true;
              break;
          }
          if (event->screenY > 160)  // happens at exit
              break;

          dX = event->screenX - penX;
          Yc = YWinSize/2;
#define TAPSLOP 7
          if (abs(dX) < TAPSLOP)
              julianDate = PalmTimeToJulian(TimGetSeconds());
          else if (dX < 0)
              julianDate += timeOffset;
          else if (dX > 0)
              julianDate -= timeOffset;

          RedrawScreen();
          handled = true;
          break;

      case frmLoadEvent:
          pfrm = FrmInitForm(event->data.frmLoad.formID);
          FrmSetEventHandler(pfrm, HandleEvent);
          FrmSetActiveForm(pfrm);
          handled = true;
          break;

      case frmOpenEvent:
          pfrm = FrmGetActiveForm();
          FrmDrawForm(pfrm);
          handled = true;
          break;

      default:
          break;
    }

#ifdef __GNUC__
    CALLBACK_EPILOGUE
#endif
    return(handled);
}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    short err;

    if (cmd == sysAppLaunchCmdNormalLaunch) {
        if ((err = StartApp(jupForm)) == 0) {
            AppLoop();
            StopApp();
        }
    }

    return(0);
}
