/*
 * palmsat.c
 * Palm main routines for satmoons program.
 *
 * Copyright 2002 by Akkana Peck.
 * You are free to use or modify this code under the Gnu Public License.
 */

#include "shallow.h"
#include "palmgrfx.h"

#include "satmoonsrsc.h"

#define YWinFake 120

double WIDESCALE = 160/3500000.;
double NARROWSCALE = 160/1000000.;
double Scale = 160/1000000.;

static int flipNS=0, flipEW=0;

extern void DrawSaturn(int Xmax, int Ymax, double Scale,
                       int flipNS, int flipEW,
                       RGBColorType* planetC, RGBColorType* skyC);

void RedrawScreen()
{
    ClearArea(0, 0, XWinSize, YWinFake);
    ChangeColor(&black);
    DateLabel(5, YWinSize-15, julianDate);
    DrawSaturn(XWinSize, YWinFake, Scale, flipNS, flipEW, &white, &black);
}

void FatalError(char* s)
{
    FrmCustomAlert(fatalAlert, s, 0, 0);
}

#define NUM_MOONS 9
struct coord_s { int x; int y; };
static struct coord_s MoonPos[NUM_MOONS];

#define LABELSLOP 4
void DrawMoon(int x, int y, int num)
{
    char numstr[2] = { num + '1', 0 };
    int xc = XWinSize/2;
    int yc = YWinFake/2;

    Line(x-1,   y,   x, y+1);
    Line(  x, y+1, x+1,   y);
    Line(x+1,   y,   x, y-1);
    Line(  x, y-1, x-1,   y);

    // Remember the coordinate
    MoonPos[num].x = x;
    MoonPos[num].y = y;

    // Put the label away from Saturn
    if (x > xc-LABELSLOP && x < xc+LABELSLOP)
        ;
    else if (x < xc) x -= 9;
    else if (x > xc) x += 2;
    if (y > yc-LABELSLOP && y < yc+LABELSLOP)
        ;
    else if (y > yc) y += 2;
    else if (y < yc) y -= 7;
    DrawString(x+2, y+7, numstr, 1);
}

/* Return 1 if we found one, else 0. */
#define TAPSLOP 7
int LookForMoon(int penX, int penY)
{
    int i;
    for (i=0; i<NumMoons; ++i)
        if (MoonPos[i].x-TAPSLOP < penX && penX < MoonPos[i].x+TAPSLOP
            &&
            MoonPos[i].y-TAPSLOP < penY && penY < MoonPos[i].y+TAPSLOP)
        {
            char buffer[30];
            int mag = satMoons[i].mag/10;
            StrPrintF(buffer, "%d: %9s (m%d.%d)",
                      i+1, satMoons[i].name,
                      mag, satMoons[i].mag - mag*10);
            Message(buffer);
            return 1;
        }
    return 0;
}

Boolean HandleCtlSelectEvent(EventPtr event)
{
    Boolean handled = true;
    unsigned long seconds;
    DateTimeType datetime;
    Word controlID = event->data.ctlSelect.controlID;

    switch(controlID) {
      case yearUp:
          seconds = JulianToPalmTime(julianDate);
          TimSecondsToDateTime(seconds, &datetime);
          ++datetime.year;
          seconds = TimDateTimeToSeconds(&datetime);
          julianDate = PalmTimeToJulian(seconds);
          break;
      case yearDown:
          seconds = JulianToPalmTime(julianDate);
          TimSecondsToDateTime(seconds, &datetime);
          --datetime.year;
          seconds = TimDateTimeToSeconds(&datetime);
          julianDate = PalmTimeToJulian(seconds);
          break;
      case dayUp:
          julianDate += 1;
          break;
      case dayDown:
          julianDate -= 1;
          break;
      case monUp:
          seconds = JulianToPalmTime(julianDate);
          TimSecondsToDateTime(seconds, &datetime);
          ++datetime.month;
          seconds = TimDateTimeToSeconds(&datetime);
          julianDate = PalmTimeToJulian(seconds);
          break;
      case monDown:
          seconds = JulianToPalmTime(julianDate);
          TimSecondsToDateTime(seconds, &datetime);
          --datetime.month;
          seconds = TimDateTimeToSeconds(&datetime);
          julianDate = PalmTimeToJulian(seconds);
          break;
      case minUp:
          julianDate += 1./24./6.;
          break;
      case minDown:
          julianDate -= 1./24./6.;
          break;
      case hourUp:
          julianDate += 1./24.;
          break;
      case hourDown:
          julianDate -= 1./24.;
          break;
      case tzUp:
          TimeZone += 1;
          TZadjustment += 1./24.;
          //julianDate += 1./24.;
          break;
      case tzDown:
          TimeZone -= 1;
          TZadjustment -= 1./24.;
          //julianDate -= 1./24.;
          break;
      case zoomBtn:
          if (Scale == WIDESCALE) {
              /*const ControlPtr btn;*/
              CtlSetLabel(event->data.ctlSelect.pControl, "z--m");
              Scale = NARROWSCALE;
          }
          else {
              CtlSetLabel(event->data.ctlSelect.pControl, "zOOm");
              Scale = WIDESCALE;
          }
          handled = 1;
          break;
      default:
          handled = false;
          break;
    }
    if (handled)
        RedrawScreen();
    return handled;
}

Boolean HandleMenuEvent(Word itemID)
{
    Boolean handled = false;
    switch (itemID)
    {
      case menuHelpAbout:
          FrmAlert(aboutAlert);
          handled = true;
          break;
      case menuHelpHelp:
          FrmAlert(helpAlert);
          handled = true;
          break;
      case menuViewFlipNS:
          flipNS = !flipNS;
          RedrawScreen();
          break;
      case menuViewFlipEW:
          flipEW = !flipEW;
          RedrawScreen();
          break;
      default:
    }
    return handled;
}

Boolean HandleEvent(EventPtr event)
{
    FormType *pfrm;
    Boolean handled = false;
    static int penX = 0;
    int penY, dX, Yc;
    static double timeOffset = 2./24.;

#ifdef __GNUC__
    CALLBACK_PROLOGUE
#endif

    switch (event->eType)
    {
      case menuEvent:
          penX = -1;
          handled = HandleMenuEvent(event->data.menu.itemID);
          break;

      case ctlSelectEvent:
          penX = -1;
          handled = HandleCtlSelectEvent(event);
          break;

      case penDownEvent:
          if (event->screenY > YWinFake)
              break;
          penX = event->screenX;
          handled = true;
          break;

      case penUpEvent:
          penY = event->screenY;
          if (penX < 0 || penY > YWinFake)   // at start, or after UI event
          {
              penX = 0;
              //printf("penX was negative, ignoring\n");
              break;
          }
          if (penX == 0)
          {
              RedrawScreen();
              handled = true;
              break;
          }
          if (event->screenY > 160)  // happens at exit
              break;

          dX = event->screenX - penX;
          Yc = YWinSize/2;
          if (abs(dX) < TAPSLOP) {
              if (LookForMoon(penX, penY))
                  break;
              julianDate = PalmTimeToJulian(TimGetSeconds());
          }
          else if ((penY < Yc && dX < 0) || (penY > Yc && dX > 0))
              julianDate -= timeOffset;
          else if ((penY < Yc && dX > 0) || (penY > Yc && dX < 0))
              julianDate += timeOffset;

          RedrawScreen();
          handled = true;
          break;

      case frmLoadEvent:
          pfrm = FrmInitForm(event->data.frmLoad.formID);
          FrmSetEventHandler(pfrm, HandleEvent);
          FrmSetActiveForm(pfrm);
          handled = true;
          break;

      case frmOpenEvent:
          pfrm = FrmGetActiveForm();
          FrmDrawForm(pfrm);
          handled = true;
          RedrawScreen();
          break;

      default:
          break;
    }

#ifdef __GNUC__
    CALLBACK_EPILOGUE
#endif
    return(handled);
}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    short err;

    if (cmd == sysAppLaunchCmdNormalLaunch) {
        if ((err = StartApp(satForm)) == 0) {
            AppLoop();
            StopApp();
        }
    }

    return(0);
}
