/*
 * shallow.h
 * Declarations of common routines for astro programs.
 * Some are in shared.c; some are in platform-specific graphics files.
 *
 * Copyright 2002 by Akkana Peck.
 * You are free to use or modify this code under the Gnu Public License.
 */

#ifdef PALMOS
#include "MathLib.h"
typedef unsigned long int TIME_T;

#else /* PALMOS */
#include <math.h>
#include <time.h>
#include <stdio.h>     // for stderr, sprintf
#include <string.h>    // for strlen

#define String char*
typedef time_t TIME_T;
#define StrLen strlen
#define StrPrintF sprintf

typedef unsigned long RGBColorType;
#endif /* PALMOS */

#define PI    3.141592654
#define TWOPI (2. * PI)
#define PI2   (PI/2.)
#define D2R   (PI/180.)

/* These variables all come from satmoons.c (all platforms) */
extern int DrawOrbits;
extern double julianDate;
extern double Inclination;
extern int NumMoons;

typedef struct satellite_s
{
    char* name;
    double sma;   /* Semimajor axis, in km */
    double ecc;   /* eccentricity */
    double pd;    /* Synodic Orbital Period in earth days */
    int mag;      /* visual magnitude *10 */
    double u0;    /* Angle from inferior geocentric conjuction
                   * measured westward along the orbit at epoch:
                   * these are guesstimated from xephem.
                   */

    /* Angle from inferior geocentric conjuction measured westward
     * (which we will calculate)
     */
    double u;
} satellite_t;

extern satellite_t satMoons[];

/*******************************************************************
 * From here down are routines which must be defined in platform
 * specific code.
 */

extern char* JulianToString(double jd);
extern void DateLabel(int x, int y, double jd);
extern void InitWindow();
extern void ClearScreen();
extern void ClearArea(int x1, int y1, int x2, int y2);
extern void DrawString(int x, int y, char* s, int len);
extern void Line(int x1, int y1, int x2, int y2);
extern void Rectangle(int x, int y, int width, int height, int fill);
extern void Oval(int xc, int yc, int r, double aspect, int fill);
extern void Arc(int xc, int yc, int r, double a1, double a2, double aspect, int fill);
extern void Message(char*);

extern void ChangeColor(RGBColorType* c);

extern void DrawMoon(int x, int y, int num);

