#include <PalmOS.h>
#include "llmgr.h"

// Get the time zone offset using LocationManager.
// See doc at http://www.star-pilot.com/locmgr/locdevel.txt

#define DEFAULT_TZ -7

extern void printf(Char* fmt, ...);
#define ERROR_DISPLAY printf

double GetTimeZoneOffset()
{
    // Get the current position
    LocalID appid = 0;
    DmSearchStateType sst;
    UInt card;
    DWord rv;
    Err err;
    locationtype loc;
    CharPtr data;	
    double timezone = 0;
    dstdatetype tzdate;
    DateTimeType datetime;

    DmGetNextDatabaseByTypeCreator(true, &sst, 'appl', 'lmgr', true,
                                   &card, &appid);
    if (!appid) {
        ERROR_DISPLAY("Could not find Location Manager; using PDT");
        return DEFAULT_TZ;
    }

    // Get Location Manager version
    err = SysAppLaunch (0, appid, 0, LAUNCH_VER, 0, &rv);
    if (err) return DEFAULT_TZ;
    if (rv < 150) {
        ERROR_DISPLAY("Need Location Manager version >=1.50; using default TZ");
        return DEFAULT_TZ;
    }

    err = SysAppLaunch (card, appid, 0, LAUNCH_GETPOS, &loc, &rv);
    if (err) return DEFAULT_TZ;
    /* At this point the variable loc has location information */

    // Get the current GMT Offset
    data = MemPtrNew (10);
    StrIToA (data, loc.tz);
    err = SysAppLaunch (card, appid, 0, LAUNCH_GETGMT, data, &rv);
    if (err) return DEFAULT_TZ;
    timezone = StrAToI(data);
    timezone /= 10.0;

    // Find out whether to observe DST:
    TimSecondsToDateTime(TimGetSeconds(), &datetime);
    tzdate.tz = loc.tz;
    tzdate.year = datetime.year;
    tzdate.month = datetime.month;
    tzdate.day = datetime.day;
    tzdate.hour = datetime.hour;
    err = SysAppLaunch (card, appid, 0, LAUNCH_ISDST, &tzdate, &rv);
    if (rv && !err)
        timezone += 1;

    return timezone;
}

