#include <time.h>

#include "shallow.h"
#include "xgrfx.h"

#include <stdlib.h>    // for exit()
#include <ctype.h>     // for isdigit()
#include <string.h>    // strchr()

double UnixTimeToJulian(time_t sec);
int parseMonth(char* mon);

static char* Usage = "Usage: %s [-o] [{julian}|{month day [year] hh:mm}]\n";

void ProcessArgs(int argc, char **argv)
{
    time_t sec;
    char* progname = argv[0];

    // Pick off flags
    if (argc > 1 && argv[1][0] == '-')
    {
        if (argv[1][1] == 'o')
        {
            DrawOrbits = 1;
            --argc;
            ++argv;
        }
        else
        {
            printf(Usage, progname);
            exit(1);
        }
            
    }

    if (argc <= 1)
    {
        time(&sec);
        julianDate = UnixTimeToJulian(sec);
        printf("Time: %s\n", ctime(&sec));
    }
    else if (argc == 2 && isdigit(argv[1][0]))
    {
        julianDate = atof(argv[1]);
        // Don't convert to Julian, assume user knows what they're doing.
        printf("Julian date argument: '%s' = %lf\n", argv[1], julianDate);
    }
    else if (argc == 4 || argc == 5)    // prog month day [year] hh:mm
    {
        struct tm then;
        char* min;
        char* hour;
        char* colon = 0;
        if (isdigit(argv[1][0]))
            then.tm_mon = atoi(argv[1]) - 1;
        else
            then.tm_mon = parseMonth(argv[1]);

        then.tm_mday = atoi(argv[2]);

        if (argc == 4) {
            then.tm_year = 102;  // default to 2002
            hour = argv[3];
        }
        else {
            then.tm_year = atoi(argv[3]);
            // A struct tm year is actually year - 1900, hah.
            if (then.tm_year > 200)
                then.tm_year -= 1900;
            hour = argv[4];
        }

        colon = /* hour +*/ strchr(hour, ':');
        if (colon >= hour) {
            min = colon;
            *(min++) = '\0';
            then.tm_min = atoi(min);
        }
        else then.tm_min = 0;
        then.tm_hour = atoi(hour);

        then.tm_sec = 0;
        then.tm_isdst = -1;

        sec = mktime(&then);
        printf("Unix time is %lu, year is %d\n", sec, then.tm_year);

        julianDate = UnixTimeToJulian(sec);
        printf("Time: %s\n", ctime(&sec));
    }
    else {
        printf(Usage, progname);
        exit(1);
    }
}

int main(int argc, char** argv)
{
    ProcessArgs(argc, argv);
    printf("Julian date is %lf\n", julianDate);

    InitWindow();

    while (HandleEvent() >= 0)
        ;
    return 0;
}

double UnixTimeToJulian(time_t sec)
{
    // Julian date for the Unix time epoch of 1970.0 is 2440587.5
    // according to http://aa.usno.navy.mil/data/docs/JulianDate.html
    double julian = sec/86400.0 + 2440587.5;
    return julian;
}

char* JulianToString(double julian)
{
    TIME_T sec = (TIME_T)((julian - 2440587.5) * 86400);
    return ctime(&sec);
}

